package wtalk.protocols.irc;

import java.io.*;
import java.net.*;
import java.util.*;

import wtalk.RMsgHandler;
import wtalk.protocols.WTalkProtocol;

public class IRC implements WTalkProtocol {
	final static int DEFAULT_PORT = 6667;

	final static String VERSION = "20030517";
	final static String NAME = "Internet Relay Chat (RFC 1459)";
	final static String AUTHOR = "Matthew Wronka <wronkm@rpi.edu>";

	protected IRCProtocol mIRCProtocol = null;
	protected Socket us = null;
	protected  PrintWriter out = null;
	protected StatusListener sl = null;
	private ReceivedListener rl = null;

	private String mServer="";
	private String mName="";
	private String mPassword="";
	private int mPort=0;

	// handles displaying of messages if !null, and not overridden;
	private RMsgHandler dmh = null;  
	// handles displaying of private messages if !null;
	private RMsgHandler sph = null;  
	// handles displaying of system messages if !null;
	private RMsgHandler smh = null;  
	// handles displaying of system messages if !null;
	private RMsgHandler rah  = null;  

	static public String getBanner() {
		return NAME + " v" + VERSION + " by " + AUTHOR;
	}

	public IRC() {
		System.err.println(getBanner());
		IRCInit();
	}

	private void IRCInit() {
		rl = new ReceivedListener(this);
		sl = new StatusListener(this);
		rl.setStatusListener(sl);
	}

	private void resetReader() {
		rl.ReportTo(dmh);
		rl.ReportPrivateTo(sph);
		rl.ReportSystemTo(smh);
		rl.ReportAutoTo(rah);
	}


	public void nickSuccess(String nick){
		mName=nick;
		RMsgHandler r = smh!=null?smh:dmh;
		if(r!=null)
			r.handle(mName,"*", "You are now known as " + nick);
	}


	/***
	 *	Below we have the implementation of the
	 *	WTalkProtocol interface.
	 ***/

	//

	// Callbacks
	//
	public void setReceivedMessageHandler(RMsgHandler r) { 
		rl.ReportTo(r); 
		dmh=r;
	}	// Default
	public void setChatMessageHandler(RMsgHandler r) { rl.ReportTo(r); }	// Chats
	public void setPrivateMessageHandler(RMsgHandler r) { 
		rl.ReportPrivateTo(r); 
		sph=r;
	} // Private
	public void setWallMessageHandler(RMsgHandler r) { rl.ReportWallTo(r); }	// Walls
	public void setSystemMessageHandler(RMsgHandler r) { 
		rl.ReportSystemTo(r); 
		smh=r;
	}	// System..

	public void setAutoMessageHandler(RMsgHandler r){ 
		rl.ReportAutoTo(r); 
		rah = r;
	}

	public void setUserListHandler(wtalk.IUserList ul){} // just here for looks

	//
	// Uncooked functions
	//
	public void sendRaw(String msg) { mIRCProtocol.SendMessage(msg); }


	//
	// Authentication/Login
	//
	public void connect() throws IOException {
		IRCInit();
		resetReader();

		us = new Socket(mServer, mPort==0? DEFAULT_PORT : mPort);

		mIRCProtocol = new IRCProtocol(us);
		rl.setIRCProtocol(mIRCProtocol);
		mIRCProtocol.Connect(mServer, mPort, mPassword, mName);

		out = new PrintWriter(us.getOutputStream(), true);
		rl.ListenTo(new BufferedReader( new InputStreamReader(us.getInputStream())));
		rl.start();
	}

	public void connect(String login) throws IOException {
		connect(mServer, mPort, login);
	}

	public void connect(String server, int port) throws IOException {
		mServer = server;	mPort = port;
		connect();
	}

	public void connect(String server, int port, String login) throws IOException {
		mName = login;
		connect(server, port);
		// send login;
	}

	public void disconnect(String msg) { 
		if(mIRCProtocol==null) return;

		mIRCProtocol.sendQuit(msg);

		mIRCProtocol=null;
	}


	public boolean isConnected() { return us!=null && us.isConnected(); }

	public void setName(String name) { 
		mName = name; 
		if(mIRCProtocol != null)
			mIRCProtocol.setNick(name);
		
	}

	public String getName() { return mName; }

	public void setPassword(String passwd) { mPassword = passwd; }
	public void changePassword(String oldpwd, String newpwd) {  }

	public void setServer(String server) { mServer = server; }
	public String getServer() { return mServer; }
	public void setServerPort(int port) { mPort = port;}
	public int getServerPort() { return mPort; }



	//
	// IM functions
	//
	public void sendMessage(String to, String data) {
		if(mIRCProtocol==null) return;


		if(!(to.charAt(0)=='&' || to.charAt(0)=='#')){
			if(sph!=null)
				sph.handle(to, null, data);
			else if(dmh!=null)
				dmh.handle(to, null, data);
		} else {
			if(dmh!=null)
				dmh.handle(to, mName, data);
		}

		mIRCProtocol.sendPrivMsg(to, data);
	}

	public void whois(String user) { 
		if(mIRCProtocol==null) return;
		mIRCProtocol.sendWhois(user);
	}

	public void addBuddy(String buddy, String group){}
	public void removeBuddy(String buddy, String group){}
	//listBuddies();

	public void setAway(String message){}

	//addDeny();
	//removeDeny();


	//
	// Chat functions
	//
	public void joinChat(String room) { 
		if(mIRCProtocol==null) return;
		mIRCProtocol.sendJoin(room);
	}

	public void partChat(String room) { 
		if(mIRCProtocol==null) return;
		mIRCProtocol.sendPart(room);
	}

	public void chatMessage(String room, String data) { sendMessage(room, data); }
	public void setTopic(String room, String topic) {
		if(mIRCProtocol==null) return;
		mIRCProtocol.sendTopic(room, topic);
	}

	public void chatList(String room) {
		if(mIRCProtocol==null) return;
		mIRCProtocol.sendList(room);
	}


	//
	// Admin functions
	//
	public void sendWall(String msg) {	sendRaw("wall " + msg); }

	public void sendSub(String to, String data){
		if(mIRCProtocol==null) return;

		StringTokenizer st = new StringTokenizer(data, " ");
		String command, text;

		if(!st.hasMoreTokens()) {
			// XXX  Error Message
		}

		command = st.nextToken();

		if(st.hasMoreTokens())
			text = data.substring(command.length()+1);
		else text = "";

		rl.CTCPHandle(command, to, getName(), text, true, false);

		mIRCProtocol.sendCTCP(to, command, text);
	}

	public void setBan(String user, boolean banned){}
}
